import { node as apiNode } from 'llparse-builder';
import Node = apiNode.Node;
export declare class Dot {
    private readonly idCache;
    private readonly ns;
    build(root: Node): string;
    private enumerateNodes;
    private buildNode;
    private buildEdgeMap;
    private buildRangeLabel;
    private buildEdgeLabel;
    private buildInvokeLabel;
    private buildOtherwiseLabel;
    private buildChar;
    private buildBuffer;
    private id;
    private escape;
}
